from fastapi import APIRouter, Depends, HTTPException
from sqlmodel import Session
from backend.models.models import Payment, User
from backend.schemas.payment import PaymentCreate, PaymentRead
from backend.database import get_session
from backend.routes.auth import get_current_user

router = APIRouter()


@router.post("/add", response_model=PaymentRead)
def add_payment(
    payment_data: PaymentCreate,
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    # Extract validated data
    payment_method = payment_data.payment_method
    card_number = payment_data.card_number
    cvv = payment_data.cvv
    expiry_date = payment_data.expiry_date

    # Create and save the payment
    new_payment = Payment(
        user_id=current_user.id,
        payment_method=payment_method,
        card_number=card_number,
        cvv=cvv,
        expiry_date=expiry_date,
    )
    session.add(new_payment)
    session.commit()
    session.refresh(new_payment)

    return new_payment


@router.get("/{payment_id}", response_model=PaymentRead)
def read_payment(
    payment_id: int,
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    """Retrieve a specific payment by ID."""
    payment = session.get(Payment, payment_id)
    if not payment or payment.user_id != current_user.id:
        raise HTTPException(status_code=404, detail="Payment not found")
    return payment


@router.put("/{payment_id}", response_model=PaymentRead)
def update_payment(
    payment_id: int,
    payment_data: PaymentCreate,
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    """Update a specific payment by ID."""
    payment = session.get(Payment, payment_id)
    if not payment or payment.user_id != current_user.id:
        raise HTTPException(status_code=404, detail="Payment not found")

    # Update payment fields
    payment.payment_method = payment_data.payment_method
    payment.card_number = payment_data.card_number
    payment.expiry_date = payment_data.expiry_date
    payment.cvv = payment_data.cvv

    session.add(payment)
    session.commit()
    session.refresh(payment)
    return payment


@router.delete("/{payment_id}", response_model=dict)
def delete_payment(
    payment_id: int,
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    """Delete a specific payment by ID."""
    payment = session.get(Payment, payment_id)
    if not payment or payment.user_id != current_user.id:
        raise HTTPException(status_code=404, detail="Payment not found")

    session.delete(payment)
    session.commit()
    return {"detail": "Payment deleted successfully"}
