from fastapi import APIRouter, Depends, HTTPException, UploadFile, File, Form
from sqlmodel import Session, func
from datetime import datetime
from backend.models.models import Product, ProductImage, User, Shop, OrderItem
from backend.schemas.product import ProductRead, ProductUpdate
from backend.database import get_session
from backend.routes.auth import get_current_user
from core.config import settings
import shutil
import os

router = APIRouter()

BACKEND_HOST = "http://127.0.0.1:8000"  # Or use settings.backend_url if available

@router.post("/create", response_model=ProductRead)
def create_product(
    name: str = Form(...),
    description: str = Form(None),
    price: float = Form(...),
    stock: int = Form(...),
    shop_id: int = Form(...),
    category_id: int = Form(None),
    images: list[UploadFile] = File(None),
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    shop = session.get(Shop, shop_id)
    if not shop or shop.owner_id != current_user.id:
        raise HTTPException(
            status_code=403, detail="Unauthorized to create product for this shop"
        )

    product = Product(
        name=name,
        description=description,
        price=price,
        stock=stock,
        shop_id=shop_id,
        category_id=category_id,
        created_at=datetime.utcnow(),
    )
    session.add(product)
    session.commit()
    session.refresh(product)

    # Directory: static/shop_{shop.name}/product_{product.name}/
    shop_dir = os.path.join(settings.static_dir, f"shop_{shop.name}")
    os.makedirs(shop_dir, exist_ok=True)
    product_dir = os.path.join(shop_dir, f"product_{product.name}")
    os.makedirs(product_dir, exist_ok=True)

    if images:
        for image in images:
            if image.filename:
                file_location = os.path.join(product_dir, image.filename)
                with open(file_location, "wb") as buffer:
                    shutil.copyfileobj(image.file, buffer)
                relative_path = os.path.relpath(file_location, settings.static_dir)
                relative_path = relative_path.replace("\\", "/")
                public_url = f"{BACKEND_HOST}/static/{relative_path}"
                product_image = ProductImage(product_id=product.id, image_url=public_url)
                session.add(product_image)
                session.commit()
    else:
        # Use default product image from static/default folder
        default_path = os.path.join(settings.static_dir, "default", "default_product.png")
        relative_path = os.path.relpath(default_path, settings.static_dir)
        relative_path = relative_path.replace("\\", "/")
        public_url = f"{BACKEND_HOST}/static/{relative_path}"
        product_image = ProductImage(product_id=product.id, image_url=public_url)
        session.add(product_image)
        session.commit()

    return product


@router.get("/list", response_model=list[ProductRead])
def read_all_products(order: str = "desc", session: Session = Depends(get_session)):
    order_by = (
        func.count(OrderItem.id).desc()
        if order == "desc"
        else func.count(OrderItem.id).asc()
    )
    products = (
        session.query(Product)
        .outerjoin(OrderItem, Product.id == OrderItem.product_id)
        .group_by(Product.id)
        .order_by(order_by)
        .all()
    )
    return products


@router.get("/get/{product_id}", response_model=ProductRead)
def read_product(product_id: int, session: Session = Depends(get_session)):
    product = session.get(Product, product_id)
    if not product:
        raise HTTPException(status_code=404, detail="Product not found")
    return product


@router.put("/put/{product_id}", response_model=ProductUpdate)
def update_product(
    product_id: int,
    name: str = Form(...),
    description: str = Form(...),
    price: float = Form(...),
    stock: int = Form(...),
    category_id: int = Form(...),
    images: list[UploadFile] = File(...),
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    product = session.get(Product, product_id)
    if not product or product.shop.owner_id != current_user.id:
        raise HTTPException(
            status_code=403, detail="Unauthorized to update this product"
        )

    product.name = name
    product.description = description
    product.price = price
    product.stock = stock
    product.category_id = category_id

    session.add(product)
    session.commit()
    session.refresh(product)

    shop = session.get(Shop, product.shop_id)
    shop_dir = os.path.join(settings.static_dir, f"shop_{shop.name}")
    os.makedirs(shop_dir, exist_ok=True)
    product_dir = os.path.join(shop_dir, f"product_{product.name}")
    os.makedirs(product_dir, exist_ok=True)

    if images:
        for file in images:
            if file.filename:
                file_location = os.path.join(product_dir, file.filename)
                with open(file_location, "wb") as buffer:
                    shutil.copyfileobj(file.file, buffer)
                relative_path = os.path.relpath(file_location, settings.static_dir)
                relative_path = relative_path.replace("\\", "/")
                public_url = f"{BACKEND_HOST}/static/{relative_path}"
                image = ProductImage(product_id=product.id, image_url=public_url)
                session.add(image)
                session.commit()

    return product


@router.delete("/delete/{product_id}")
def delete_product(
    product_id: int,
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    product = session.get(Product, product_id)
    if not product or product.shop.owner_id != current_user.id:
        raise HTTPException(
            status_code=403, detail="Unauthorized to delete this product"
        )
    session.delete(product)
    session.commit()
    return {"message": "Product deleted successfully"}
